from .color import Colors, ContinuousColors
from . import matplotlib
from .multi_plots import MultiPlots
from . import watcher
from . import tensorboard

__all__ = ["Colors", "ContinuousColors", "MultiPlots", "matplotlib", "watcher", "tensorboard"]

try:
    from . import seaborn
except ImportError:
    pass
else:
    __all__.append("seaborn")



from numpy import ndarray

from .abstract_face_detector import AbstractFaceDetector
from ..image import rgb2gray
from ...backend import VisualLib


class OpencvFaceDetector(AbstractFaceDetector):

    def __init__(self):
        try:
            import cv2
        except ImportError:
            raise ImportError("OpenCV is not installed. Please install it first.")

        self.face_cascade = cv2.CascadeClassifier(cv2.data.haarcascades + "haarcascade_frontalface_default.xml")

    def detect_face(self, img: ndarray, *args, **kwargs):
        # img should be gray scale
        if len(img.shape) == 3:
            img = rgb2gray((img * 255).astype(int), backend=VisualLib.OPENCV)

        faces = self.face_cascade.detectMultiScale(img, *args, **kwargs)
        return faces

    def detect_face_batch(self, imgs: ndarray, *args, **kwargs):
        return [self.detect_face(img, *args, **kwargs) for img in imgs]



from abc import ABC, abstractmethod


class AbstractFaceDetector(ABC):

    @abstractmethod
    def detect_face(self, img):
        pass

    @abstractmethod
    def detect_face_batch(self, imgs):
        pass



from ._ffmpeg_check import ffmpeg_available
from .video import frames2video
from .image import rgb2gray, rgb2gray_batch

__all__ = ['ffmpeg_available', 'frames2video', 'rgb2gray', 'rgb2gray_batch']

if ffmpeg_available:
    from .ffmpeg import ffmpeg_command, video2frames, merge_video_audio, resample_video_fps, mp32wav

    __all__ += ['ffmpeg_command', 'video2frames', 'merge_video_audio', 'resample_video_fps', 'mp32wav']



from enum import Enum
from typing import Optional


class VisualLib(Enum):
    OPENCV = 1
    PYTORCH = 2
    MATPLOTLIB = 3
    PIL = 4
    FFMPEG = 5
    SKIMAGE = 6

    _is_opencv_available: Optional[bool] = None
    _is_torchvision_available: Optional[bool] = None
    _is_matplotlib_available: bool = True  # already included in the requirements.txt
    _is_pil_available: Optional[bool] = None
    _is_ffmpeg_available: bool = None
    _is_skimage_available: Optional[bool] = None

    @classmethod
    def opencv_available(cls) -> bool:
        if cls._is_opencv_available is None:
            try:
                import cv2
                cls._is_opencv_available = True
            except ImportError:
                cls._is_opencv_available = False
        return cls._is_opencv_available

    @classmethod
    def pytorch_available(cls) -> bool:
        if cls._is_torchvision_available is None:
            try:
                import torchvision
                cls._is_torchvision_available = True
            except ImportError:
                cls._is_torchvision_available = False
        return cls._is_torchvision_available

    @classmethod
    def matplotlib_available(cls) -> bool:
        return cls._is_matplotlib_available

    @classmethod
    def pil_available(cls) -> bool:
        if cls._is_pil_available is None:
            try:
                import PIL
                cls._is_pil_available = True
            except ImportError:
                cls._is_pil_available = False
        return cls._is_pil_available

    @classmethod
    def ffmpeg_available(cls) -> bool:
        if cls._is_ffmpeg_available is None:
            from ..preprocess._ffmpeg_check import ffmpeg_available
            cls._is_ffmpeg_available = ffmpeg_available
        return cls._is_ffmpeg_available

    @classmethod
    def skimage_available(cls) -> bool:
        if cls._is_skimage_available is None:
            try:
                import skimage
                cls._is_skimage_available = True
            except ImportError:
                cls._is_skimage_available = False
        return cls._is_skimage_available



from typing import TypeVar

from .bus import EventBus


class EventMeta(type):

    def __call__(cls, *args, bus=EventBus.default, **kwargs):
        event = super().__call__(*args, **kwargs)
        bus.emit(event)
        return event


class Event(metaclass=EventMeta):
    bus: EventBus


E = TypeVar("E", bound=Event)
